/**
 * Created by janis dähne.
 */
import {
  EditableExercisePreviewFromBackend,
  GroupPreviewExerciseTupleFrontendOnly
} from "../../../types/exercisePreview";
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionBase, ActionType} from "./groupExercisesSiteActionTypes";
import {AllActions as GetGroupExercisesActions, reducer as getGroupExercisesReducer} from './getGroupExercisesReducer'
import {
  AllActions as GetGroupsForExercisesActions,
  reducer as getGroupsForExercisesCrudReducer
} from './getGroupsForExercisesCrudReducer'
import {
  AllActions as DeleteGroupExerciseActions,
  reducer as deleteGroupExerciseReducer
} from './deleteGroupExerciseReducer'
import {FrontendSettingsManager} from "../../../helpers/frontendSettingsManager";
import {UserGroupBase, UserGroupFromBackend} from '../../../types/group'
import Logger from '../../../helpers/logger'
import {ActionTypeGlobal, RESET_GlobalAction} from '../globalActions'

export type State = {


  readonly groups: ReadonlyArray<UserGroupFromBackend>
  readonly groupExercises: ReadonlyArray<GroupPreviewExerciseTupleFrontendOnly>
  readonly isLoading: boolean

  readonly isTagsFilterDisplayed: boolean
}


export const initial: State = {

  groups: [],
  groupExercises: [],
  isLoading: false,

  isTagsFilterDisplayed: false,
}


//---pagination
export interface SetSearchTextAction extends ActionBase {
  readonly type: ActionType.setSearchText
  readonly searchText: string
  readonly groupId: number
}

export interface SetActivePaginationPageAction extends ActionBase {
  readonly type: ActionType.setPaginationActivePage
  readonly activePage: number
  readonly groupId: number
}

export interface SetPaginationPageSizeAction extends ActionBase {
  readonly type: ActionType.setPaginationPageSize
  readonly pageSize: number
  readonly groupId: number
}

//---pagination end

export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof EditableExercisePreviewFromBackend
  readonly sortDirection: boolean | null
  readonly groupId: number
}

export interface SET_isTagsFilterDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isTagsFilterDisplayed
  readonly isTagsFilterDisplayed: boolean
}

export interface SET_isLoadingManageGroupExercisesSiteACtion extends ActionBase {
  readonly type: ActionType.SET_isLoadingManageGroupExercisesSite
  readonly isLoading: boolean
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction

  | SET_sortByKeyAction
  | SET_isTagsFilterDisplayedAction
  | SetSearchTextAction
  | SetActivePaginationPageAction
  | SetPaginationPageSizeAction
  | SET_isLoadingManageGroupExercisesSiteACtion

  | GetGroupExercisesActions
  | DeleteGroupExerciseActions
  | GetGroupsForExercisesActions

  | RESET_GlobalAction

export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {


    case ActionType.SET_isLoadingManageGroupExercisesSite:

      return {
        ...state,
        isLoading: action.isLoading
      }

    case ActionType.SET_isTagsFilterDisplayed:

      FrontendSettingsManager.setGroupExercisesSiteTagsFilterIsDisplayed(action.isTagsFilterDisplayed)

      return {
        ...state,
        isTagsFilterDisplayed: action.isTagsFilterDisplayed
      }

    case ActionType.GET_groupsForExercises_PENDING:
    case ActionType.GET_groupsForExercises_FULFILLED:
    case ActionType.GET_groupsForExercises_REJECTED:
      return getGroupsForExercisesCrudReducer(state, action)

    case ActionType.GET_groupExercises_PENDING:
    case ActionType.GET_groupExercises_FULFILLED:
    case ActionType.GET_groupExercises_REJECTED:
      return getGroupExercisesReducer(state, action)

    case ActionType.DELETE_groupExercise_PENDING:
    case ActionType.DELETE_groupExercise_FULFILLED:
    case ActionType.DELETE_groupExercise_REJECTED:
      return deleteGroupExerciseReducer(state, action)

    case ActionType.SET_sortByKey:

      const groupTuple = state.groupExercises.find(p => p.userGroup.id === action.groupId)

      if (!groupTuple) {
        Logger.error('could not find group in group exercises')
      }

      return {
        ...state,
        groupExercises: state.groupExercises.map(value =>
                                                   value.userGroup.id === action.groupId
                                                   ? {
                                                       ...value,
                                                       pagination: {
                                                         ...value.pagination,
                                                         orderByProp: action.sortByKey,
                                                         isOrderByDesc: action.sortDirection
                                                       }
                                                     }
                                                   : value
        )
      }

    case ActionType.setSearchText:
      return {
        ...state,
        groupExercises: state.groupExercises.map(value =>
                                                   value.userGroup.id === action.groupId
                                                   ? {
                                                       ...value,
                                                       pagination: {
                                                         ...value.pagination,
                                                         searchText: action.searchText
                                                       }
                                                     }
                                                   : value
        )
      }
    case ActionType.setPaginationActivePage:
      return {
        ...state,
        groupExercises: state.groupExercises.map(value =>
                                                   value.userGroup.id === action.groupId
                                                   ? {
                                                       ...value,
                                                       pagination: {
                                                         ...value.pagination,
                                                         currentPage: action.activePage
                                                       }
                                                     }
                                                   : value
        )
      }

    case ActionType.setPaginationPageSize:

      FrontendSettingsManager.set_groupExercisesPageSize(action.pageSize, action.groupId)

      return {
        ...state,
        groupExercises: state.groupExercises.map(value =>
                                                   value.userGroup.id === action.groupId
                                                   ? {
                                                       ...value,
                                                       pagination: {
                                                         ...value.pagination,
                                                         pageSize: action.pageSize,
                                                         currentPage: 1,
                                                       }
                                                     }
                                                   : value
        )
      }

    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial,
        isTagsFilterDisplayed: FrontendSettingsManager.getFrontendSettings().groupExercisesSiteTagsFilterIsDisplayed,
      }

    default:
      notExhaustive(action)
      return state
  }
}

